/* ptsetvardb.p
 *
 * prompt the user for various settings that might require a db connection
 *
 */

{lib/protop.i}

define temp-table tt_CSC
  field uNum as integer
  index uNum-idx is unique primary uNum
.

session:add-super-procedure( this-procedure ).

subscribe to "protop_clearCSC"       anywhere run-procedure "clearCSC".
subscribe to "protop_getUserNum"     anywhere run-procedure "getUserNum".
subscribe to "protop_getUserPID"     anywhere run-procedure "getUserPID".
subscribe to "protop_getLockLimit"   anywhere run-procedure "getLockLimit".
subscribe to "protop_getUserTblName" anywhere run-procedure "getUserTblName".
subscribe to "protop_getUserIdxName" anywhere run-procedure "getUserIdxName".

return.


procedure clearCSC:

  define buffer updConnect for _Connect.

  define variable i  as integer no-undo.
  define variable ok as logical no-undo.
  define variable t  as character no-undo.

  if session:batch = yes then return.

  for each tt_CSC:
    i = i + 1.
  end.

  if i = 0 and lastkey = asc( "_" ) then	/* only show this if the user explicitly asked to clear the cache -- IOW they typed "_"		*/
    do:
      message
        skip(1)
        "    You have not enabled the client statement cache for any users in this ProTop session.    " skip
        skip(1)
       view-as alert-box information
        title " Clear Client Statement Cache "
      .
    end.

  if i = 0 then return.				/* perhaps we are quitting and did not show the message above and there is nothing to do...	*/

  ok = yes.

  message color red
    skip(1)
    substitute( "Clear the &1 session&2 that you enabled the Client Statement Cache for? ", ( if i > 1 then trim( string( i, ">>>>>>" )) else "" ), ( if i > 1 then "s" else "" )) skip
    skip(1)
                "This isn't strictly required but in most cases it is best to proactively" skip
                "clean these up rather than leave the sessions enabled.                  " skip
    skip(1)
    view-as alert-box question buttons yes-no
    title " Clear Client Statement Cache "
    update ok
  .
  if ok = no then return.

  for each tt_CSC:

      do for updConnect transaction:
        find updConnect exclusive-lock where _Connect-id = uNum + 1 no-error.
        if available( updConnect ) then
          do:
            buffer updConnect:handle:buffer-field( "_connect-cachingType" ):buffer-value = 0 no-error.
            delete tt_CSC.
          end.
      end.
    
  end.

  return.

end.


procedure getUserNum:

  define input-output parameter userNum as integer no-undo label "  usrNum" format ">>>>9".
  define input-output parameter userPID as integer no-undo.

  define buffer updConnect for _Connect.

  define variable cType as integer no-undo format "9".

  userNum_loop: do while true on error undo, leave
    on endkey undo, leave:

    cType = ?.

    if userNum > 0 and userNum <> ? then
      do for updConnect:
        find updConnect no-lock where _Connect-id = userNum + 1 no-error.
        if available( updConnect ) then
          cType = buffer updConnect:handle:buffer-field( "_connect-cachingType" ):buffer-value no-error.
      end.

    update
      skip(1)
      '  Enter the DB Connection Number for the session of interest. This is the  ' skip
      '  "Usr#" column found on many ProTop screens, in the DB .lg file, and in   ' skip
      '  many Progress error messages.  ' skip
      skip(1)
      '  Usr#' userNum skip
      skip(1)
      '  Client statement cache type:' cType '     0 = Off'             skip
      '                                      1 = Single (Top of Stack)' skip
      '                                      2 = Full Call Stack'       skip
      skip(1)
      '      Most of the time you want "1". You typically do not need the full   ' skip
      '      stack unless you are planning to look at the detailed User Info     ' skip
      '      screen (the "U" command). The full stack option obviously takes     ' skip
      '      more memory, uses more space in the client/server networking        ' skip
      '      communications and is also more likely to use the scratch disk      ' skip
      '      (resulting in extraneous IO operations).                            ' skip
      skip(1)
     with
      frame getVar
      title " DB Connection Number "
      row 3
      column ( current-window:width-chars - 78 ) / 2
      width 78
/*
      column ( {&PTWIDTH} - 92 ) / 2
      width 92
 */
      no-labels
      overlay
    .

    if userNum > 0 and userNum <> ? then
      do for updConnect transaction:
        find updConnect exclusive-lock where _Connect-id = userNum + 1 no-error.
        if available( updConnect ) and _Connect-usr <> ? then
          do:
            if cType = 1 or cType = 2 then
              do:
                find tt_CSC where tt_CSC.uNum = _Connect-usr no-error.
                if not available tt_CSC then
                  do:
                    create tt_CSC.
                    tt_CSC.uNum = _Connect-usr.
                  end.
                buffer updConnect:handle:buffer-field( "_connect-cachingType" ):buffer-value = cType no-error.
              end.
             else if cType = 0 then
              do:
                find tt_CSC where tt_CSC.uNum = _Connect-usr no-error.
                if available tt_CSC then
                  delete tt_CSC.
                buffer updConnect:handle:buffer-field( "_connect-cachingType" ):buffer-value = 0 no-error.
              end.
          end.
         else
          do:
            message "No such usrNum:" userNum.
            next userNum_loop.
          end.
      end.

    leave.

  end.

  hide frame getVar.

  userPID = ?.

  return.

end.


procedure getUserPID:

  define input-output parameter userPID as integer no-undo label "  usrPID" format ">>>>>>>>>9".
  define input-output parameter userNum as integer no-undo.

  define buffer updConnect for _Connect.

  define variable cType as integer no-undo format "9".

  do on error undo, leave
    on endkey undo, leave:

    cType = ?.

    find _Connect no-lock where _Connect-PID = userPID no-error.
    if available( _connect ) then
      cType = buffer _connect:handle:buffer-field( "_connect-cachingType" ):buffer-value no-error.

    update
      skip(1)
      '  Enter the DB Connection PID for the session of interest. This is the ' skip
      '  "PID" column on many ProTop screens, in the DB .lg file, some error  ' skip
      '  messges or from OS commands such as "ps".    ' skip
      skip(1)
      '  usrPID:' userPID skip
      skip(1)
      '  Client statement cache type:' cType '  0 = Off, 1 = Single, 2 = Stack' skip
      '                                      (Most of the time you want "1")' skip
      skip(1)
     with
      frame getVar
      title " DB Connection Process ID "
      row 3
/*
      column ( {&PTWIDTH} - 96 ) / 2
      width 96
 */
      column ( current-window:width-chars - 78 ) / 2
      width 78
      no-labels
      overlay
    .

    if userPID > 1 and userPID <> ? then
      do for updConnect transaction:
        find first updConnect exclusive-lock where _Connect-PID = userPID no-error.
        if available( updConnect ) then
          do:
            if cType = 1 or cType = 2 then
              do:
                find tt_CSC where uNum = _Connect-usr no-error.
                if not available tt_CSC then
                  do:
                    create tt_CSC.
                    uNum = _Connect-usr.
                  end.
                buffer updConnect:handle:buffer-field( "_connect-cachingType" ):buffer-value = cType no-error.
              end.
             else if cType = 0 then
              do:
                find tt_CSC where uNum = _Connect-usr no-error.
                if available tt_CSC then
                  delete tt_CSC.
                buffer updConnect:handle:buffer-field( "_connect-cachingType" ):buffer-value = 0 no-error.
              end.
          end.
      end.

  end.

  hide frame getVar.

  userNum = ?.

  return.

end.


procedure getLockLimit:

  define input-output parameter lockLimit as integer no-undo format "->>>>9" label "  lockLimit".

  define variable newLockLimit as integer no-undo format "->>>>9" label "  lockLimit".

  define variable ok as logical no-undo.

  assign
    ok = no
    newLockLimit = lockLimit
  .

  lock_limit_upd: do on error undo, leave lock_limit_upd
    on endkey undo, leave lock_limit_upd:

    update
      skip(1)
      '  Non-default values are DANGEROUS! This enables _Lock scanning for ' skip
      '  blocked sessions. ' skip
      skip(1)
      newLockLimit skip
      skip(1)
      '  The lockLimit is the maximum number of _Lock entries to examine '   skip
      '  before giving up when attempting to determine who else might have ' skip
      '  a record lock.  This *might* be attempted for every user -- so '    skip
      '  the total number of attempts could be as high as -n * lockLimit.'   skip
      skip(1)
      '  This can be useful but it needs to be used with extreme caution. '  skip
      '  Scanning the _Lock table is very costly.  Especially on busy '      skip
      '  production systems with -L set to large values.  The use of this '  skip
      '  feature is strongly discouraged.'                                   skip
      skip(1)
      '  If you are not *very* sure that you need this do NOT change it.'    skip
      skip(1)
     with
      frame getVar
      title " Lock Limit "
      row 3
/*
      column ( {&PTWIDTH} - 100 ) / 2
      width 100
 */
      column ( current-window:width-chars - 78 ) / 2
      width 78
      side-labels
      overlay
    .

    if newLockLimit <> lockLimit then			/* the user is attempting to change the lock limit, discourage them	*/
      do on error undo, leave lock_limit_upd
        on endkey undo, leave lock_limit_upd:

        message color red
          skip
          'You have been warned that this is a bad idea.' skip
          'Are you SURE that you want to ignore that advice?'
          skip(1)
         view-as
          alert-box question buttons yes-no
          title " This Is A Very Bad Idea "
          update ok					/* F1 = ^X = F4 = ^E = <return> = don't use MESSAGE statements...	*/
        .

      end.

    leave lock_limit_upd.

  end.

  hide frame getVar.

  if ok = yes then lockLimit = newLockLimit.

  return.

end.


procedure getUserTblName:

  define input-output parameter userTblName as character no-undo format "x(50)".
  define input-output parameter userTblNum  as integer   no-undo.

  define variable reqString as character no-undo.
  define variable tNum      as integer   no-undo.

  getTbl: do on error undo, leave getTbl
    on endkey undo, leave getTbl:

    update
      skip(1)
      '  Track top users of:' skip
      skip(1)
      userTblName
      skip(1)
      '  Enter a valid table name or number.  Use "" or ? to clear table tracking.' skip
      skip(1)
     with
      frame getVar
      title " Table Name "
      row 3
/*
      column ( {&PTWIDTH} - 100 ) / 2
      width 100
 */
      column ( current-window:width-chars - 78 ) / 2
      width 78
      side-labels
      overlay
    .

    if userTblName = "" or userTblName = ? then
      do:
        userTblName = "".
        userTblNum = -999999.
        leave getTbl.
      end.

    find tt_tbl where tt_tbl.tblName = userTblName no-error.
    if available tt_tbl then
      do:
        userTblName = tt_tbl.tblName.
        userTblNum = tt_tbl.xid.
        leave getTbl.
      end.

    tNum = ?.
    tNum = integer( userTblName ) no-error.
    if tNum <> ? then
      do:
        find tt_tbl where tt_tbl.xid = tNum no-error.
        if available tt_tbl then
          do:
            userTblName = tt_tbl.tblName.
            userTblNum = tt_tbl.xid.
            leave getTbl.
          end.
      end.

    message "Please enter a valid table name or number.".
    pause.
    undo, retry getTbl.

  end.

  hide frame getVar.

  if userTblName = "" or userTblName = ? or userTblNum = -999999 then
    do:
      publish "protop_getReqStr" ( input-output reqString ).
   /* message substitute( "[&1] [&2] [&3] [&4] [&5]", userTblName, userTblName, userTblNum, reqString, lookup( "UsersOfTable", reqString )).	*/
   /* pause.	*/
      if lookup( "UsersOfTable", reqString ) > 0 then
        publish "protop_show" ( "usersoftbl" ).
    end.

  return.

end.


procedure getUserIdxName:

  define input-output parameter userIdxName as character no-undo format "x(50)".
  define input-output parameter userIdxNum  as integer   no-undo.

  define variable reqString as character no-undo.
  define variable iNum      as integer   no-undo.

  getIdx: do on error undo, leave getIdx
    on endkey undo, leave getIdx:

    update
      skip(1)
      '  Track top users of:' skip
      skip(1)
      userIdxName
      skip(1)
      '  Enter a valid index name (table.index) or number.  Use "" or ? to clear ' skip
      '  index tracking.' skip
      skip(1)
     with
      frame getVar
      title " Index Name "
      row 3
/*
      column ( {&PTWIDTH} - 100 ) / 2
      width 100
 */
      column ( current-window:width-chars - 78 ) / 2
      width 78
      side-labels
      overlay
    .

    if userIdxName = "" or userIdxName = ? then
      do:
        userIdxName = "".
        userIdxNum = -999999.
        leave getIdx.
      end.

    if num-entries( userIdxName, "." ) = 2 then
      do:
        find tt_idx where tt_idx.tblName = entry( 1, userIdxName, "." ) and tt_idx.idxName = entry( 2, userIdxName, "." ) no-error.
        if available tt_idx then
          do:
            userIdxName = substitute( "&1.&2", tt_idx.tblName, tt_idx.idxName ).
            userIdxNum = tt_idx.xid.
            leave getIdx.
          end.
      end.

    iNum = ?.
    iNum = integer( userIdxName ) no-error.
    if iNum <> ? then
      do:
        find tt_idx where tt_idx.xid = iNum no-error.
        if available tt_idx then
          do:
            userIdxName = substitute( "&1.&2", tt_idx.tblName, tt_idx.idxName ).
            userIdxNum = tt_idx.xid.
            leave getIdx.
          end.
      end.

    message "Please enter a valid index name or number.".
    pause.
    undo, retry getIdx.

  end.

  hide frame getVar.

  if userIdxName = "" then
    do:
      publish "protop_getReqStr" ( input-output reqString ).
      if lookup( "UsersOfIndexActivity", reqString ) > 0 then
        publish "protop_show" ( "usersofidx" ).
    end.

  return.

end.

